{
  Solutions to the Advent Of Code.
  Copyright (C) 2024  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UPolynomialRootsTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, UPolynomial, UPolynomialRoots, UBigInt;

type

  { TPolynomialRootsTestCase }

  TPolynomialRootsTestCase = class(TTestCase)
  protected
    FRootIsolation: TRootIsolation;
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestBisectionRootIsolation;
  end;

implementation

{ TPolynomialRootsTestCase }

procedure TPolynomialRootsTestCase.SetUp;
begin
  inherited SetUp;
  FRootIsolation := TRootIsolation.Create;
end;

procedure TPolynomialRootsTestCase.TearDown;
begin
  FRootIsolation.Free;
  inherited TearDown;
end;

procedure TPolynomialRootsTestCase.TestBisectionRootIsolation;
var
  a: TBigIntPolynomial;
  r: Int64;
begin
  // y = 3 * (x - 34000) * (x - 23017) * (x - 5) * (x^2 - 19) * (x + 112)
  //   = 3 * x^6 - 170730 * x^5 + 2329429920 * x^4 + 251300082690 * x^3 - 1270471872603 * x^2 + 4774763204640 * x - 24979889760000
  a := TBigIntPolynomial.Create([-24979889760000, 4774763204640, -1270471872603, 251300082690, 2329429920, -170730, 3]);
  r := FRootIsolation.Bisect(a);
  AssertEquals(0, r);
end;

initialization

  RegisterTest(TPolynomialRootsTestCase);
end.

